"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const React = require("react");
const abab_1 = require("abab");
const ServerDataContext = React.createContext();
exports.ServerDataContext = ServerDataContext;
class ServerDataStore {
    constructor() {
        this.dataLoaders = {};
        this.isFirstRun = true;
    }
    /** Queues an action to be run either immedietly (client) or after render (server) */
    runAction(name, ...args) {
        if (!ServerDataStore.isServer) {
            return;
        }
        const b64 = name + (abab_1.btoa(JSON.stringify(args)) || "");
        if (!this.dataLoaders[b64]) {
            this.dataLoaders[b64] = {
                name,
                args,
            };
        }
    }
    /** Server side version, for client version use the static method
     *  Runs a queued action and returns the result
     */
    getResult(name, ...args) {
        return __awaiter(this, void 0, void 0, function* () {
            return ServerDataStore.actions[name](...args);
        });
    }
    /** Client side version, for server version use the context method
    *   Runs a queued action and returns the result, or fetches from head data if already present
    */
    static getResult(name, ...args) {
        return __awaiter(this, void 0, void 0, function* () {
            const b64 = name + (abab_1.btoa(JSON.stringify(args)) || "");
            if (global.__pushData && global.__pushData[b64] !== undefined) {
                return global.__pushData[b64];
            }
            return ServerDataStore.actions[name](...args);
        });
    }
    getDataAsString() {
        return __awaiter(this, void 0, void 0, function* () {
            const promises = Object.keys(this.dataLoaders).map((key) => {
                return ServerDataStore.actions[this.dataLoaders[key].name](...this.dataLoaders[key].args);
            });
            const data = yield Promise.all(promises);
            return `<script>
    window.__pushData = {}
        ${Object.keys(this.dataLoaders).map((key, index) => {
                return `window.__pushData['${key}'] = ${JSON.stringify(data[index]).replace(/</g, `\\u003c`)}`;
            })}
</script>`;
        });
    }
    getTagsAsString() {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.tagRenderer) {
                return this.tagRenderer.renderTags();
            }
            return "";
        });
    }
    static registerAction(name, resolver) {
        ServerDataStore.actions[name] = resolver;
    }
    registerTagRenderer(renderer) {
        if (ServerDataStore.isServer) {
            this.tagRenderer = renderer;
        }
    }
    get firstRun() {
        const val = this.isFirstRun;
        this.isFirstRun = false;
        return val;
    }
}
ServerDataStore.actions = {};
ServerDataStore.isServer = !window.window;
exports.ServerDataStore = ServerDataStore;
//# sourceMappingURL=ServerDataContext.js.map