"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const ServerDataContext_1 = require("./ServerDataContext");
exports.ServerDataContext = ServerDataContext_1.ServerDataContext;
exports.ServerDataStore = ServerDataContext_1.ServerDataStore;
const ServerDataTag_1 = require("./ServerDataTag");
exports.ServerDataTag = ServerDataTag_1.default;
const react_1 = require("react");
function useServerData(name, ...args) {
    return react_1.useMemo(() => {
        const store = react_1.useContext(ServerDataContext_1.ServerDataContext);
        if (store) {
            store.runAction(name, ...args);
            const isServer = !window.window;
            let runner;
            if (!isServer) {
                const resource = {
                    state: 0,
                    result: undefined,
                    error: undefined,
                    read: () => {
                        switch (resource.state) {
                            case 0:
                                throw runner;
                            case 1:
                                return resource.result;
                            case 2:
                                throw resource.error;
                        }
                    }
                };
                const runner = new Promise((resolve, reject) => __awaiter(this, void 0, void 0, function* () {
                    try {
                        const result = yield ServerDataContext_1.ServerDataStore.getResult(name, ...args);
                        resource.state = 1;
                        resource.result = result;
                        resolve(result);
                    }
                    catch (e) {
                        resource.state = 2;
                        resource.error = e;
                        reject(e);
                    }
                }));
                return resource;
            }
            else {
                // Server gets a dummy object until react.suspense
                return {
                    read: () => undefined,
                };
            }
        }
    }, [name, ...args]);
}
exports.useServerData = useServerData;
//# sourceMappingURL=index.js.map